from typing import List, Any
from tabulate import tabulate
from .loggingConsts import MAX_VALUE_LENGTH_IN_TABLES


def makeTable(
    data: List[List[str]],
    headers: List[str] = None,
    label: str = None,
    maxValueLength: int = MAX_VALUE_LENGTH_IN_TABLES,
) -> str:
    kwargs = {
        "tablefmt": "grid",
        "stralign": "left",
        "numalign": "left",
    }
    for row in data:
        for cellIdx, cellVal in enumerate(row):
            row[cellIdx] = truncateStr(cellVal, maxValueLength)
    if headers is None:
        table = tabulate(
            data,
            **kwargs,
        )
    else:
        table = tabulate(
            data,
            headers=headers,
            **kwargs,
        )
    s = "" if label is None else f"{label}:\n"
    return f"{s}{table}"


def truncateStr(s: Any, max_length: int) -> str:
    s = str(s)
    if len(s) > max_length - 3:
        return s[: max_length - 3] + "..."
    return s



import traceback
from datetime import datetime
from typing import Callable, Tuple


def exceptionToStr(e: Exception) -> str:
    current_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    error_message = f"Exception was thrown at {current_time}:\n{''.join(traceback.format_exception(None, e, e.__traceback__))}"
    return error_message


def getErrorHandlingDecorators(
    onError: Callable[[str], None]
) -> Tuple[Callable, Callable]:
    def handleError(func):
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                error_message = exceptionToStr(e)
                onError(error_message)

        return wrapper

    def handleErrorAsync(func):
        async def wrapper(*args, **kwargs):
            try:
                return await func(*args, **kwargs)
            except Exception as e:
                error_message = exceptionToStr(e)
                onError(error_message)

        return wrapper

    return handleError, handleErrorAsync



import time
from datetime import datetime, timezone, timedelta


# create timeIt decorator
def timeIt(func):
    def wrapper(*args, **kwargs):
        start_time = time.time()
        result = func(*args, **kwargs)
        timeTaken = round(time.time() - start_time, 4)
        print(f"function {func.__name__} took {timeTaken} seconds")
        return result

    return wrapper


def secondsSinceTimestamp(timestamp: float) -> float:
    current_time = time.time()
    elapsed_time = current_time - timestamp
    return elapsed_time


def secondsSinceDatetime(dt: datetime) -> float:
    current_time = datetime.now(timezone.utc) if dt.tzinfo else datetime.now()

    elapsed_time = (current_time - dt).total_seconds()
    return elapsed_time


def subtractSecondsFromDatetime(dt: datetime, seconds: int) -> datetime:
    return dt - timedelta(seconds=seconds)


def datetimeToStr(dt: datetime) -> str:
    return dt.strftime("%Y-%m-%d %H:%M:%S")


def timestampToStr(ts: float) -> str:
    return datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H:%M:%S")
